#cobra.solvers.cplex_solver
#Interface to ilog/cplex 12.4 python interface

from os import name as __name
from copy import deepcopy
from warnings import warn
###solver specific parameters
from .parameters import status_dict, variable_kind_dict, \
     sense_dict, parameter_mappings, parameter_defaults, \
     objective_senses, default_objective_sense

from ..core.Solution import Solution

from time import time
solver_name = 'cplex'
parameter_defaults = parameter_defaults[solver_name]
sense_dict = eval(sense_dict[solver_name])


parameter_mappings = parameter_mappings[solver_name]
objective_senses = objective_senses[solver_name]
from cplex import Cplex, SparsePair
class Problem(Cplex):
    def __init__(self):
        Cplex.__init__(self)
__solver_class = Problem
variable_kind_dict = eval(variable_kind_dict[solver_name])
status_dict = eval(status_dict[solver_name])
def get_status(lp):
    status = lp.solution.get_status_string().lower()
    if status in status_dict:
        status = status_dict[status]
    else:
        status = 'failed'
    return status

def get_objective_value(lp):
    return lp.solution.get_objective_value()

def format_solution(lp, cobra_model, **kwargs):
    status = get_status(lp)
    #TODO: It might be able to speed this up a little.
    if status in ('optimal', 'time_limit'):
        objective_value = lp.solution.get_objective_value()
        #This can be sped up a little
        x_dict = dict(zip(lp.variables.get_names(),
                     lp.solution.get_values()))
        x = lp.solution.get_values()
        #MIP's don't have duals
        if lp.get_problem_type() in (Cplex.problem_type.MIQP,
                                     Cplex.problem_type.MILP):

            y = y_dict = None
        else:
            y_dict = dict(zip(lp.linear_constraints.get_names(),
                              lp.solution.get_dual_values()))
            y = lp.solution.get_dual_values()
    else:
        x = y = x_dict = y_dict = objective_value = None

    the_solution = Solution(objective_value, x=x, x_dict=x_dict,
                            status=status, y=y, y_dict=y_dict)
    return the_solution    

def set_parameter(lp, parameter_name, parameter_value):
    """
    
    """
    if parameter_name == 'objective.set_sense':
        if parameter_value in objective_senses:
            parameter_value = eval(objective_senses[parameter_value])
    try:
        eval('lp.%s(%s)'%(parameter_name, repr(parameter_value)))
    except Exception, e:
        print "Couldn't set parameter %s: %s"%(parameter_name, repr(e))



def create_problem(cobra_model, quadratic_component=None, **kwargs):
    """Solver-specific method for constructing a solver problem from
    a cobra.Model.  This can be tuned for performance using kwargs


    """
    the_parameters = parameter_defaults
    if kwargs:
        the_parameters = deepcopy(parameter_defaults)
        the_parameters.update(kwargs)

    lp = __solver_class()
    if 'log_file' not in the_parameters:
        lp.set_results_stream(None)
        lp.set_warning_stream(None)
    [set_parameter(lp, parameter_mappings[k], v)
     for k, v in the_parameters.iteritems() if k in parameter_mappings]
    if 'relax_b' in the_parameters:
        warn('need to reimplement relax_b')
        relax_b = False
    else:
        relax_b = False

    #Using the new objects
    #NOTE: This might be slow
    objective_coefficients = []
    lower_bounds = []
    upper_bounds = []
    variable_names = []
    variable_kinds = []
    [(objective_coefficients.append(x.objective_coefficient),
      lower_bounds.append(x.lower_bound),
      upper_bounds.append(x.upper_bound),
      variable_names.append(x.id),
      variable_kinds.append(variable_kind_dict[x.variable_kind]))
     for x in cobra_model.reactions]
    #Cplex decides that the problem is a MIP if variable_kinds are supplied
    #even if there aren't any integers.
    if variable_kind_dict['integer'] in variable_kinds:
        lp.variables.add(obj=objective_coefficients,
                         lb=lower_bounds,
                         ub=upper_bounds,
                         names=variable_names,
                         types=variable_kinds)
    else:
        lp.variables.add(obj=objective_coefficients,
                         lb=lower_bounds,
                         ub=upper_bounds,
                         names=variable_names)

   ## if relax_b:
        ## range_values = zeros(len(cobra_model.metabolites))
        ## b_values = array([x._bound for x in cobra_model.metabolties])
        ## for the_nonzero in list(b_values.nonzero()[0]):
        ##     range_values[the_nonzero] = -relax_b

    constraint_sense = []
    constraint_names = []
    constraint_limits = []
    [(constraint_sense.append(x._constraint_sense),
      constraint_names.append(x.id),
      constraint_limits.append(x._bound))
     for x in cobra_model.metabolites]

    the_linear_expressions = []
    #NOTE: This won't work with metabolites that aren't in any reaction
    for the_metabolite in cobra_model.metabolites:
        variable_list = []
        coefficient_list = []
        for the_reaction in the_metabolite._reaction:
            variable_list.append(the_reaction.id)
            coefficient_list.append(the_reaction._metabolites[the_metabolite])
        the_linear_expressions.append(SparsePair(ind=variable_list,
                                                 val=coefficient_list))
    # Set objective to quadratic program
    if quadratic_component is not None:
        set_quadratic_objective(lp, quadratic_component)

    if relax_b:
        lp.linear_constraints.add(lin_expr=the_linear_expressions,
                                  rhs=constraint_limits,
                                  range_values=list(range_values),
                                  senses=constraint_sense,
                                  names=constraint_names)

    else:
        lp.linear_constraints.add(lin_expr=the_linear_expressions,
                                  rhs=constraint_limits,
                                  senses=constraint_sense,
                                  names=constraint_names)

    #Set the problem type as cplex doesn't appear to do this correctly
    problem_type = Cplex.problem_type.LP
    if Cplex.variables.type.integer in variable_kinds:
        if quadratic_component is not None:
            problem_type = Cplex.problem_type.MIQP
        else:
            problem_type = Cplex.problem_type.MILP
    elif quadratic_component is not None:
        problem_type = Cplex.problem_type.QP
    lp.set_problem_type(problem_type)
    return(lp)


def set_quadratic_objective(lp, quadratic_objective):
    if not hasattr(quadratic_objective, 'todok'):
        raise Exception('quadratic component must have method todok')

    # Reset the quadratic coefficient if it exists
    if lp.objective.get_num_quadratic_nonzeros() > 0:
        lp.objective.set_quadratic((0.,) * lp.variables.get_num())

    quadratic_component_scaled = quadratic_objective.todok()

    lp.parameters.emphasis.numerical.set(1)
    for k, v in quadratic_component_scaled.items():
        lp.objective.set_quadratic_coefficients(int(k[0]), int(k[1]), v)

def change_variable_bounds(lp, index, lower_bound, upper_bound):
    lp.variables.set_lower_bounds(index, lower_bound)
    lp.variables.set_upper_bounds(index, upper_bound)

def change_variable_objective(lp, index, objective):
    lp.objective.set_linear(index, objective)


def change_coefficient(lp, met_index, rxn_index, value):
    lp.linear_constraints.set_coefficients(met_index, rxn_index, value)

def update_problem(lp, cobra_model, **kwargs):
    """A performance tunable method for updating a model problem file

    lp: A cplex problem object

    cobra_model: the cobra.Model corresponding to 'lp'

    """
    #When reusing the basis only assume that the objective coefficients or bounds can change
    #BUG with changing / unchanging the basis

    try:
        update_problem_reaction_bounds = kwargs['update_problem_reaction_bounds']
    except:
        update_problem_reaction_bounds = True
    try:
        quadratic_component = kwargs['quadratic_component']
        if quadratic_component is not None:
            warn("update_problem does not yet take quadratic_component as a parameter")
    except:
        quadratic_component = None

    if new_objective is not None:
        lp.objective.set_linear([(x.id, float(x.objective_coefficient))
                                 for x in cobra_model.reactions])
    if update_problem_reaction_bounds:
        lp.variables.set_upper_bounds([(x.id, float(x.upper_bound))
                                        for x in cobra_model.reactions])
        lp.variables.set_lower_bounds([(x.id, float(x.lower_bound))
                                        for x in cobra_model.reactions])



###
def solve_problem(lp, **kwargs):
    """A performance tunable method for solving a problem

    """
    #Update parameter settings if provided
    if kwargs:
        [set_parameter(lp, parameter_mappings[k], v)
         for k, v in kwargs.iteritems() if k in parameter_mappings]
    try:
        the_problem = kwargs['the_problem']
    except:
        the_problem = False
    if isinstance(the_problem, __solver_class):
        try:
            the_basis = the_problem.solution.basis.get_basis()
            lp.start.set_basis(the_basis[0],the_basis[1])
            lp.parameters.preprocessing.presolve.set(0)
        except:
            warn("cplex_java isn't yet configured to reuse the basis")
    
    lp.solve()
    #If the solver takes more than 0.1 s with a hot start it is likely stuck
    status = get_status(lp)
    return status

    
def solve(cobra_model, **kwargs):
    """

    """
    #Start out with default parameters and then modify if
    #new onese are provided
    the_parameters = deepcopy(parameter_defaults)
    if kwargs:
        the_parameters.update(kwargs)
    for i in ["new_objective", "update_problem", "the_problem"]:
        if i in the_parameters:
            raise Exception("Option %s removed" % i)
    if 'error_reporting' in the_parameters:
        warn("error_reporting deprecated")


    lp = create_problem(cobra_model, **the_parameters)

    ###Try to solve the problem using other methods if the first method doesn't work
    try:
        lp_method = the_parameters['lp_method']
    except:
        lp_method = 1
    the_methods = [1, 2, 3, 4, 5, 6]
    if lp_method in the_methods:
        the_methods.remove(lp_method)
    #Start with the user specified method
    the_methods.insert(0, lp_method)
    for the_method in the_methods:
        the_parameters['lp_method'] = the_method
        try:
            status = solve_problem(lp, **the_parameters)
        except:
            status = 'failed'
        if status == 'optimal':
            break

    the_solution = format_solution(lp, cobra_model)
    #if status != 'optimal':
    #    print '%s failed: %s'%(solver_name, status)
    #cobra_model.solution = the_solution
    #solution = {'the_problem': lp, 'the_solution': the_solution}
    return the_solution



"""bindings to the glpk solver through pyglpk"""
# because both this module and the library are named glpk
from __future__ import absolute_import
from glpk import LPX

from ..core.Solution import Solution


# mappers from cobra representation to glpk
variable_kind_dict = {
    'continuous': float,
    'integer': int}
# mappers from glpk representations to cobra
status_dict = {
    'opt': 'optimal',
    'nofeas': 'infeasible',
    'unbnd': 'unbounded'}


def create_problem(cobra_model, objective_sense="maximize", lp=None):
    if lp is None:
        lp = LPX()  # Create empty problem instance
        lp.name = cobra_model.id
        lp.rows.add(len(cobra_model.metabolites))
        lp.cols.add(len(cobra_model.reactions))

    if objective_sense == 'maximize':
        lp.obj.maximize = True
    elif objective_sense == 'minimize':
        lp.obj.maximize = False
    else:
        raise ValueError("objective_sense not 'maximize' or 'minimize'")

    # create metabolites/constraints as rows
    for i, r in enumerate(lp.rows):
        metabolite = cobra_model.metabolites[i]
        r.name = metabolite.id
        b = float(metabolite._bound)
        c = metabolite._constraint_sense
        # constraint sense is set by changing the bounds
        if c == 'E':
            r.bounds = (b, b)
        elif c == 'L':
            r.bounds = (None, b)
        elif c == 'G':
            r.bounds = (b, None)
        else:
            raise ValueError("%s is not a valid constraint_sense" % c)

    # create reactions/variables as columns
    for i, c in enumerate(lp.cols):
        reaction = cobra_model.reactions[i]
        c.name = reaction.id
        c.kind = variable_kind_dict[reaction.variable_kind]
        c.bounds = (reaction.lower_bound, reaction.upper_bound)
        lp.obj[i] = float(reaction.objective_coefficient)

    # create S matrix
    lp.matrix = [(int(i), int(j), c) \
        for (i, j), c in cobra_model.to_array_based_model().S.todok().iteritems()]
    return lp


def solve_problem(lp):
    lp.simplex()
    try:
        status = status_dict[lp.status]
    except:
        status = "unknown error: " + str(lp.status)
    solution = Solution(status)
    solution.status = status
    if status == 'optimal':
        solution.objective_value = lp.obj.value
        solution.x_dict = dict((c.name, c.primal) for c in lp.cols)
        # return the duals as well as the primals for LPs
        if lp.kind == "float":
            solution.y_dict = dict((c.name, c.dual) for c in lp.cols)
        else:
            solution.y_dict = None
    return solution


def solve(cobra_model, objective_sense="maximize", **kwargs):
    return solve_problem(create_problem(cobra_model, objective_sense))



try:
    import setuptools
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
from setuptools import setup, find_packages
from sys import argv

import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from cobra.version import get_version

__version = get_version(pep440=True)
setup_kwargs = {}

# for running parallel tests due to a bug in python 2.7.3
# http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing
except:
    None

# cython is optional for building. The c file can be used directly. However,
# to run sdist, the c file must be generated, which requires cython.
try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None
    if "sdist" in argv:
        raise Exception("cython required for sdist")

# for building the cglpk solver
try:
    from distutils.extension import Extension
    from distutils.command.build_ext import build_ext
    from os.path import isfile, abspath, dirname
    from platform import system

    class FailBuild(build_ext):
        """allow building of the C extension to fail"""
        def run(self):
            try:
                build_ext.run(self)
            except Exception as e:
                warn(e)

        def build_extension(self, ext):
            try:
                build_ext.build_extension(self, ext)
            except:
                None

    build_args = {}
    if system() == "Darwin":  # otherwise Mac Clang gives errors
        build_args["extra_compile_args"] = ["-Qunused-arguments"]
    build_args["libraries"] = ["glpk"]
    setup_kwargs["cmdclass"] = {"build_ext": FailBuild}
    # To statically link libglpk to the built extension, add the glpk.h header
    # and static library libglpk.a to the build directory. A static libglpk.a
    # can be built by running configure with the export CLFAGS="-fPIC" and
    # copying the file from src/.libs
    if isfile("libglpk.a"):
        build_args["library_dirs"] = [dirname(abspath("libglpk.a"))]
    if isfile("glpk.h"):
        build_args["include_dirs"] = [dirname(abspath("glpk.h"))]
    # use cython if present, otherwise use c file
    if cythonize:
        ext_modules = cythonize([Extension("cobra.solvers.cglpk",
                ["cobra/solvers/cglpk.pyx"], **build_args)])
    else:
        ext_modules = [Extension("cobra.solvers.cglpk",
                ["cobra/solvers/cglpk.c"], **build_args)]
except:
    ext_modules = None

extras = {
    'parallel': ['pp>=1.6.0'],
    'matlab': ["mlabwrap>=1.1"],
    'sbml': ["python-libsbml-experimental"],
    'array': ["numpy>=1.6", "scipy>=11.0"],
    'display': ["matplotlib", "brewer2mpl", "pandas"]
}

all_extras = set()
for extra in extras.values():
    all_extras.update(extra)
extras["all"] = list(all_extras)

setup(
    name = "cobra",
    version = __version,
    packages = find_packages(exclude=['cobra.oven', 'cobra.oven*']),
    #scripts = [''],
    #put in numpy, scipy, libsbml, and pyglpk
    setup_requires = [],
    #install_requires = ['numpy>=1.6', 'scipy>=0.10'],
    #leave blank because it tries to build scipy/numpy on os x when they are
    #installed by the superpack.  And these are not really essential for core functions.
    install_requires = [],
    extras_require = extras,
    ext_modules = ext_modules,

    package_data = {
         '': ['test/data/*',
              'VERSION',
              'mlab/matlab_scripts/*m']},

    author = "Daniel Robert Hyduke <danielhyduke@gmail.com>, Ali Ebrahim <aebrahim@ucsd.edu>",
    author_email = "danielhyduke@gmail.com",
    description = "COBRApy is a package for constraints-based modeling of biological networks",
    license = "GPL V3.0",
    keywords = "metabolism biology linear programming optimization flux balance analysis fba",
    url = "https://github.com/opencobra/cobrapy",
    test_suite = "cobra.test.suite",
    long_description = "COnstraint-Based Reconstruction and Analysis (COBRA) methods are widely used for genome-scale modeling of metabolic networks in both prokaryotes and eukaryotes.  COBRApy is a constraint-based modeling package that is designed to accomodate the biological complexity of the next generation of COBRA models and provides access to commonly used COBRA methods, such as flux balance analysis, flux variability analysis, and gene deletion analyses.  Through the mlabwrap module it is possible to use COBRApy to call many additional COBRA methods present in the COBRA Toolbox for MATLAB.",
    download_url = 'https://pypi.python.org/pypi/cobra',
    classifiers = ['Development Status :: 5 - Production/Stable',
                   'Environment :: Console',
                   'Intended Audience :: Science/Research',
                   'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
                   'Operating System :: MacOS :: MacOS X',
                   'Operating System :: Microsoft :: Windows :: Windows 7',
                   'Operating System :: Microsoft :: Windows :: Windows Vista',
                   'Operating System :: Microsoft :: Windows :: Windows XP',
                   'Operating System :: POSIX',
                   'Operating System :: POSIX :: Linux',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.3',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: Implementation :: CPython',
                   'Programming Language :: Python :: Implementation :: Jython',
                   'Topic :: Scientific/Engineering',
                   'Topic :: Scientific/Engineering :: Bio-Informatics'
                   ],
    platforms = "Python >= 2.6 on GNU/Linux, Mac OS X >= 10.7, Microsoft Windows >= 7. \n Jython >= 2.5 on Java >= 1.6",
    **setup_kwargs
    )



from warnings import catch_warnings
from tempfile import gettempdir
from os import unlink
from os.path import join, split
from unittest import TestCase, TestLoader, TextTestRunner, skipIf
from functools import partial
import sys

if __name__ == "__main__":
    from cobra import io
    from cobra.test import data_directory, create_test_model
    from cobra.test import ecoli_mat, ecoli_pickle, ecoli_json
    from cobra.test import salmonella_sbml, salmonella_pickle
    from cobra.test import salmonella_fbc_sbml
else:
    from .. import io
    from . import data_directory, create_test_model
    from . import ecoli_mat, ecoli_pickle, ecoli_json
    from . import salmonella_sbml, salmonella_pickle
    from . import salmonella_fbc_sbml

libraries = ["scipy", "libsbml"]
for library in libraries:
    try:
        exec("import %s" % library)
    except ImportError:
        exec("%s = None" % library)


class TestCobraIO(object):
    def compare_models(self, model1, model2):
        self.assertEqual(len(model1.reactions),
                         len(model2.reactions))
        self.assertEqual(len(model1.metabolites),
                         len(model2.metabolites))
        for attr in ("id", "name", "lower_bound", "upper_bound"):
            self.assertEqual(getattr(model1.reactions[0], attr),
                             getattr(model2.reactions[0], attr))
            self.assertEqual(getattr(model1.reactions[10], attr),
                             getattr(model2.reactions[10], attr))
            self.assertEqual(getattr(model1.reactions[-1], attr),
                             getattr(model2.reactions[-1], attr))
        for attr in ("id", "name"):
            self.assertEqual(getattr(model1.metabolites[0], attr),
                             getattr(model2.metabolites[0], attr))
            self.assertEqual(getattr(model1.metabolites[10], attr),
                             getattr(model2.metabolites[10], attr))
            self.assertEqual(getattr(model1.metabolites[-1], attr),
                             getattr(model2.metabolites[-1], attr))
        self.assertEqual(len(model1.reactions[0].metabolites),
                         len(model2.reactions[0].metabolites))
        self.assertEqual(len(model1.reactions[20].metabolites),
                         len(model2.reactions[20].metabolites))
        self.assertEqual(len(model1.reactions[-1].metabolites),
                         len(model2.reactions[-1].metabolites))
        # ensure they have the same solution max
        model1.optimize()
        model2.optimize()
        self.assertAlmostEqual(model1.solution.f, model2.solution.f,
                places=3)
    def test_read(self):
        #with catch_warnings(record=True) as w:
        read_model = self.read_function(self.test_file)
        self.compare_models(self.test_model, read_model)
   
    def test_read_nonexistent(self):
        # make sure that an error is raised when given a nonexistent file
        self.assertRaises(IOError, self.read_function, "fake_file")

    def test_write_and_reread(self):
        test_output_filename = join(gettempdir(), split(self.test_file)[-1])
        self.write_function(self.test_model, test_output_filename)
        #with catch_warnings(record=True) as w:
        reread_model = self.read_function(test_output_filename)
        self.compare_models(self.test_model, reread_model)
        unlink(test_output_filename)

@skipIf(not libsbml, "libsbml required")
class TestCobraIOSBML(TestCase, TestCobraIO):
    def setUp(self):
        self.test_model = create_test_model()
        self.test_file = salmonella_sbml
        self.read_function = io.read_sbml_model
        self.write_function = io.write_sbml_model

@skipIf(not libsbml, "libsbml required")
class TestCobraIOSBMLfbc(TestCase, TestCobraIO):
    def setUp(self):
        self.test_model = create_test_model()
        self.test_file = salmonella_fbc_sbml
        self.read_function = io.read_sbml_model
        self.write_function = partial(io.write_sbml_model,
                                      use_fbc_package=True)

@skipIf(not scipy, "scipy required")
class TestCobraIOmat(TestCase, TestCobraIO):
    def setUp(self):
        self.test_model = create_test_model(ecoli_pickle)
        self.test_file = ecoli_mat
        self.read_function = io.load_matlab_model
        self.write_function = io.save_matlab_model


class TestCobraIOjson(TestCase, TestCobraIO):
    def setUp(self):
        self.test_model = create_test_model(ecoli_pickle)
        self.test_file = ecoli_json
        self.read_function = io.load_json_model
        self.write_function = io.save_json_model

# make a test suite to run all of the tests
loader = TestLoader()
suite = loader.loadTestsFromModule(sys.modules[__name__])

def test_all():
    TextTestRunner(verbosity=2).run(suite)

if __name__ == "__main__":
    test_all()



#cobra.manipulation.modify.py
import re
from copy import deepcopy

from warnings import warn

def prune_unused_metabolites(cobra_model):
    """Removes metabolites that aren't involved in any reactions in the model

    cobra_model: A Model object.

    """
    inactive_metabolites = []
    active_metabolites = []
    for the_metabolite in cobra_model.metabolites:
        if len(the_metabolite._reaction) == 0:
            the_metabolite.remove_from_model(cobra_model)
            inactive_metabolites.append(the_metabolite)
        else:
            active_metabolites.append(the_metabolite)
    if inactive_metabolites:
        return inactive_metabolites
    else:
        warn('All metabolites used in at least 1 reaction')

    
def prune_unused_reactions(cobra_model):
    """Removes reactions from cobra_model.  

    cobra_model: A Model object.

    reactions_to_prune: None, a string matching a reaction.id, a cobra.Reaction, or
    as list of the ids / Reactions to remove from cobra_model.
    If None then the function will delete reactions that
    have no active metabolites in the model.

    """
    pruned_reactions = []
    reactions_to_prune = [x for x in cobra_model.reactions
                          if len(x._metabolites) == 0]
    for the_reaction in reactions_to_prune:
        try:
            the_reaction.remove_from_model(cobra_model)
            pruned_reactions.append(the_reaction)
        except:
            warn('%s not in %s' % (the_reaction.id, cobra_model.id))
    if not pruned_reactions:
        warn('All reactions have at least 1 metabolite')
        return


def undelete_model_genes(cobra_model):
    """Undoes the effects of a call to delete_model_genes. Modifies cobra_model in place.

    cobra_model:  A cobra.Model object

    """

    if cobra_model._trimmed_genes is not None:
        [setattr(x, 'functional', True)
         for x in cobra_model._trimmed_genes]
    
    if cobra_model._trimmed_reactions is not None:
        for the_reaction, (lower_bound,
                           upper_bound) in cobra_model._trimmed_reactions.items():
            the_reaction.lower_bound = lower_bound
            the_reaction.upper_bound = upper_bound
    #
    cobra_model._trimmed_genes = None
    cobra_model._trimmed_reactions = None
    cobra_model._trimmed = False
    #Reset these to deal with potential bugs from users accessing private variables
    for the_attribute in  ['_lower_bounds', '_upper_bounds',
                           '_S', '_objective_coefficients']:
        if hasattr(cobra_model, the_attribute):
            setattr(cobra_model, the_attribute, None)

spontaneous_re = re.compile('(^|(?<=( |\()))s0001(?=( |\)|$))')
def find_gene_knockout_reactions(cobra_model, gene_list):
    """identify reactions which will be disabled when the genes are knocked out"""

    potential_reactions = set()
    for x in gene_list:
        potential_reactions.update(x._reaction)

    knocked_out_reactions = []
    for the_reaction in potential_reactions:
        # operate on a copy
        gene_reaction_rule = "" + the_reaction.gene_reaction_rule
        for the_gene in the_reaction._genes:
            if the_gene in gene_list:
                gene_reaction_rule = gene_reaction_rule.replace(the_gene.id, 'False')
            else:
                gene_reaction_rule = gene_reaction_rule.replace(the_gene.id, 'True')
        gene_reaction_rule = spontaneous_re.sub('True', gene_reaction_rule)
        if not eval(gene_reaction_rule):
            knocked_out_reactions.append(the_reaction)
    return knocked_out_reactions


def delete_model_genes(cobra_model, gene_list,
                       cumulative_deletions=False, disable_orphans=False):
    """delete_model_genes will set the upper and lower bounds for reactions
    catalyzed by the genes in gene_list if deleting the genes means that
    the reaction cannot proceed according to
    cobra_model.reactions[:].gene_reaction_rule

    cumulative_deletions: False or True.  If True then any previous
    deletions will be maintained in the model.

    TODO: Rewrite to use dicts for _trimmed*

    TODO: All this will be replaced by Boolean logic associated with
    #the cobra.Gene.functional in cobra.Reaction.gene_reaction_rule

    TODO: Update this to refer to cobra.(Gene|Reaction) in the
    _trimmed_(genes|reactions) fields and remove _trimmed_indices
    
    """
    if not hasattr(cobra_model, '_trimmed') or not cumulative_deletions:
        cobra_model._trimmed = False
        cobra_model._trimmed_genes = []
        cobra_model._trimmed_reactions = {} #Store the old bounds in here.
    spontaneous_re = re.compile('(^|(?<=( |\()))s0001(?=( |\)|$))')
    #Allow a single gene to be fed in as a string instead of a list.
    if not hasattr(gene_list, '__iter__') or \
           hasattr(gene_list, 'id'):  #cobra.Gene has __iter__
        gene_list = [gene_list]

    if not hasattr(gene_list[0], 'id'):
        if gene_list[0] in cobra_model.genes:
                tmp_gene_dict = dict([(x.id, x) for x in cobra_model.genes])
        else:
            #assume we're dealing with names if no match to an id
            tmp_gene_dict = dict([(x.name, x) for x in cobra_model.genes])
        gene_list = [tmp_gene_dict[x] for x in gene_list]

    #Make the genes non-functional
    for x in gene_list:
        x.functional = False

    for the_reaction in find_gene_knockout_reactions(cobra_model, gene_list):
        old_lower_bound = the_reaction.lower_bound
        old_upper_bound = the_reaction.upper_bound
        cobra_model._trimmed_reactions[the_reaction] = (old_lower_bound,
                                                        old_upper_bound)
        the_reaction.lower_bound = 0.
        the_reaction.upper_bound = 0.
        cobra_model._trimmed = True

    cobra_model._trimmed_genes =  list(set(cobra_model._trimmed_genes + gene_list))



if __name__ == '__main__':
    from time import time
    from cobra.test import create_test_model
    cobra_model = create_test_model()


    #TODO: Add in tests for each function
    cumulative_deletions=False
    disable_orphans=False
    gene_list = ['STM1067', 'STM0227']
    #The following reactions are trimmed when STM1332 and STM1101 are deleted
    dependent_reactions = set(['3HAD121',
                               '3HAD160',
                               '3HAD80',
                               '3HAD140',
                               '3HAD180',
                               '3HAD100',
                               '3HAD181',
                               '3HAD120',
                               '3HAD60',
                               '3HAD141',
                               '3HAD161',
                               'T2DECAI',
                               '3HAD40'])
    delete_model_genes(cobra_model, gene_list)
    symmetric_difference = dependent_reactions.symmetric_difference([x.id for x in cobra_model._trimmed_reactions])
    if len(symmetric_difference) == 0:
        'Successful deletion of %s'%repr(gene_list)
    else:
        'Failed deletion of %s\n%s reactions did not match'%(repr(gene_list),
                                                                   repr(symmetric_difference))









from __future__ import with_statement, absolute_import
import sys
from os import name as __name
available_tests = ['unit_tests', 'solvers', 'flux_analysis', 'io_tests']

del __name

from os.path import abspath as __abspath
from os.path import join as __join
from os.path import split as __split
from os.path import sep as __sep

cobra_directory = __abspath(__join(__split(__abspath(__file__))[0], ".."))
cobra_location = __abspath(__join(cobra_directory, ".."))
data_directory = __join(__split(__abspath(__file__))[0], "data")
if not data_directory.endswith(__sep):
    data_directory += __sep

salmonella_sbml = __join(data_directory, "salmonella.xml")
salmonella_fbc_sbml = __join(data_directory, "salmonella_fbc.xml")
salmonella_pickle = __join(data_directory, "salmonella.pickle")
salmonella_reaction_p_values_pickle = __join(data_directory, "salmonella_reaction_p_values.pickle")
ecoli_sbml = __join(data_directory, "iJO1366.xml")
ecoli_pickle = __join(data_directory, "iJO1366.pickle")
ecoli_mat = __join(data_directory, "iJO1366.mat")
ecoli_json = __join(data_directory, "iJO1366.json")
yersinia_sbml = __join(data_directory, 'Yersinia_pestis_CO92_iPC815.xml')
yersinia_pickle = __join(data_directory, 'Yersinia_pestis_CO92_iPC815.pickle')

__test_pickles = {'Salmonella_enterica': salmonella_pickle,
                  'Escherichia_coli': ecoli_pickle,
                  'Yersinia_pestis': yersinia_pickle}
__test_xml = {'Salmonella_enterica': salmonella_sbml,
              'Escherichia_coli': ecoli_sbml,
              'Yersinia_pestis': yersinia_sbml}
del __abspath, __join, __split, __sep

def create_test_model(test_pickle=salmonella_pickle):
    """Returns a cobra model for testing.  The default model is the up to date
    version of the Salmonella enterica Typhimurium LT2 model published in
    Thiele et al. 2011 BMC Sys Bio 5:8

    test_pickle: The complete file name of a pickled cobra.Model or SBML XML
    file to be read.  We currently provide Salmonella enterica Typhimurium
    and Escherichia coli models whose paths are stored in cobra.test.salmonella_pickle
    and cobra.test.ecoli_pickle, respectively.  The ecoli model is a variant of the
    model published in Orth et al. 2011 Mol Syst Biol 7:535

    """
    from os import name as __name
    try:
        from cPickle import load
    except:
        from pickle import load

    try: 
        with open(test_pickle, "rb") as infile:
            model = load(infile)
    except:
        #if the pickle can't be loaded then load the sbml xml
        from warnings import warn
        warn("Couldn't load %s.  Loading the default model %s instead"%(test_pickle,
                                                                        salmonella_sbml))
        sys.path.insert(0, cobra_location)
        from cobra.io import read_sbml_model
        model = read_sbml_model(salmonella_sbml)
        sys.path.pop(0)
    return model

def create_test_suite():
    """create a unittest.TestSuite with available tests"""
    from unittest import TestLoader, TestSuite
    loader = TestLoader()
    suite = TestSuite()
    for test_name in available_tests:
        exec("from . import " + test_name)
        suite.addTests(loader.loadTestsFromModule(eval(test_name)))
    return suite

suite = create_test_suite()

def test_all():
    """###running unit tests on cobra py###"""
    from unittest import TextTestRunner
    TextTestRunner(verbosity=2).run(create_test_suite())
